@@ -1,6 +1,8 @@
 
+from copy import copy
 from email.message import EmailMessage
 from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Union
+import warnings
 
 import jinja2
 from redmail.email.attachment import Attachments
@@ -32,16 +34,18 @@ class EmailSender:
         SMTP host address.
     port : int
         Port to the SMTP server.
-    user_name : str, optional
+    username : str, optional
         User name to authenticate on the server.
     password : str, optional
         User password to authenticate on the server.
     cls_smtp : smtplib.SMTP
         SMTP class to use for connection. See options 
-        from `Python smtplib docs <https://docs.python.org/3/library/smtplib.html>`_.
+        from :stdlib:`Python smtplib docs <smtplib.html>`.
     use_starttls : bool
         Whether to use `STARTTLS <https://en.wikipedia.org/wiki/Opportunistic_TLS>`_ 
         when connecting to the SMTP server.
+    user_name : str, optional
+        Deprecated alias for username. Please use username instead.
     **kwargs : dict
         Additional keyword arguments are passed to initiation in ``cls_smtp``.
         These are stored as attribute ``kws_smtp``
@@ -75,6 +79,10 @@ class EmailSender:
     html_template : str
         Name of the template to use as the HTML body of emails 
         if not specified in the send method.
+    use_jinja : bool
+        Use Jinja to render text/HTML. If Jinja is disabled,
+        images cannot be embedded to HTML, templates have no
+        effect and body_params are not used. Defaults True
     templates_html : jinja2.Environment
         Jinja environment used for loading HTML templates
         if ``html_template`` is specified in send.
@@ -96,6 +104,10 @@ class EmailSender:
     kws_smtp : dict
         Keyword arguments passed to ``cls_smtp``
         when connecting to the SMTP server.
+    connection : smtplib.SMTP, None
+        Connection to the SMTP server. Created and closed
+        before and after sending each email unless there 
+        is an existing connection.
 
     Examples
     --------
@@ -127,11 +139,16 @@ class EmailSender:
 
     attachment_encoding = 'UTF-8'
 
-    def __init__(self, host:str, port:int, user_name:str=None, password:str=None, cls_smtp:smtplib.SMTP=smtplib.SMTP, use_starttls:bool=True, **kwargs):
+    def __init__(self, host:str, port:int, username:str=None, password:str=None, cls_smtp:smtplib.SMTP=smtplib.SMTP, use_starttls:bool=True, **kwargs):
+
+        if "user_name" in kwargs and username is None:
+            warnings.warn("Argument user_name was renamed as username. Please use username instead.", FutureWarning)
+            username = kwargs.pop("user_name")
+
         self.host = host
         self.port = port
 
-        self.user_name = user_name
+        self.username = username
         self.password = password
 
         # Defaults
@@ -145,11 +162,14 @@ class EmailSender:
         self.html = None
         self.html_template = None
         self.text_template = None
+        self.use_jinja = True
 
         self.use_starttls = use_starttls
         self.cls_smtp = cls_smtp
         self.kws_smtp = kwargs
         
+        self.connection = None
+
     def send(self,
              subject:Optional[str]=None,
              sender:Optional[str]=None,
@@ -208,6 +228,9 @@ class EmailSender:
             DataFrames.
         body_params : dict, optional
             Extra Jinja parameters passed to the HTML and text bodies.
+        use_jinja : bool
+            Use Jinja to render text/HTML. If Jinja is disabled, body content cannot be 
+            embedded, templates have no effect and body parameters do nothing.
         attachments : dict, optional
             Attachments of the email. If dict value is string, the attachment content
             is the string itself. If path, the attachment is the content of the path's file.
@@ -226,7 +249,7 @@ class EmailSender:
                 email = EmailSender(
                     host='localhost', 
                     port=0, 
-                    user_name='me@example.com', 
+                    username='me@example.com', 
                     password='<PASSWORD>'
                 )
                 email.send(
@@ -281,7 +304,8 @@ class EmailSender:
                   body_images:Optional[Dict[str, Union[str, bytes, 'plt.Figure', 'Image']]]=None, 
                   body_tables:Optional[Dict[str, 'pd.DataFrame']]=None, 
                   body_params:Optional[Dict[str, Any]]=None,
-                  attachments:Optional[Dict[str, Union[str, os.PathLike, 'pd.DataFrame', bytes]]]=None) -> EmailMessage:
+                  attachments:Optional[Dict[str, Union[str, os.PathLike, 'pd.DataFrame', bytes]]]=None,
+                  use_jinja=None) -> EmailMessage:
         """Get the email message"""
 
         subject = subject or self.subject
@@ -295,6 +319,7 @@ class EmailSender:
         text = text or self.text
         html_template = html_template or self.html_template
         text_template = text_template or self.text_template
+        use_jinja = self.use_jinja if use_jinja is None else use_jinja
 
         if subject is None:
             raise ValueError("Email must have a subject")
@@ -306,11 +331,14 @@ class EmailSender:
             cc=cc,
             bcc=bcc,
         )
-
-        if text is not None or text_template is not None:
+        has_text = text is not None or text_template is not None
+        has_html = html is not None or html_template is not None
+        has_attachments = attachments is not None
+        if has_text:
             body = TextBody(
                 template=self.get_text_template(text_template),
                 table_template=self.get_text_table_template(),
+                use_jinja=use_jinja
             )
             body.attach(
                 msg, 
@@ -319,10 +347,11 @@ class EmailSender:
                 jinja_params=self.get_text_params(extra=body_params, sender=sender),
             )
 
-        if html is not None or html_template is not None:
+        if has_html:
             body = HTMLBody(
                 template=self.get_html_template(html_template),
                 table_template=self.get_html_table_template(),
+                use_jinja=use_jinja
             )
             body.attach(
                 msg,
@@ -331,6 +360,14 @@ class EmailSender:
                 tables=body_tables,
                 jinja_params=self.get_html_params(extra=body_params, sender=sender)
             )
+        
+        self._set_content_type(
+            msg,
+            has_text=has_text,
+            has_html=has_html,
+            has_attachments=has_attachments,
+        )
+
         if attachments:
             att = Attachments(attachments, encoding=self.attachment_encoding)
             att.attach(msg)
@@ -350,7 +387,7 @@ class EmailSender:
 
     def get_sender(self, sender:Union[str, None]) -> str:
         """Get sender of the email"""
-        return sender or self.sender or self.user_name
+        return sender or self.sender or self.username
 
     def _create_body(self, subject, sender, receivers=None, cc=None, bcc=None) -> EmailMessage:
         msg = EmailMessage()
@@ -366,17 +403,47 @@ class EmailSender:
             msg['bcc'] = bcc
         return msg
 
+    def _set_content_type(self, msg:EmailMessage, has_text, has_html, has_attachments):
+
+        # NOTE: we don't convert emails that have only text/plain to multiplart/mixed
+        # in order to keep the messages minimal (as often desired with simple plain text)
+
+        if has_html or has_attachments:
+            # Change the structure to multipart/mixed if possible.
+            # This seems to be the most versatile and most unproblematic top level content-type
+            # as otherwise content may be missing or it may be misrendered.
+            # See: https://stackoverflow.com/a/23853079/13696660
+            # See issues: #23, #37
+            msg.make_mixed()
+
     def send_message(self, msg:EmailMessage):
         "Send the created message"
-
-        server = self.connect()
-        server.send_message(msg)
-        
-        server.quit()
+        if self.is_alive:
+            self.connection.send_message(msg)
+        else:
+            # The connection was opened for this message
+            # thus it is also closed with this message
+            with self:
+                self.connection.send_message(msg)
     
-    def connect(self) -> smtplib.SMTP:
+    def __enter__(self):
+        self.connect()
+
+    def __exit__(self, *args):
+        self.close()
+
+    def connect(self):
         "Connect to the SMTP Server"
-        user = self.user_name
+        self.connection = self.get_server()
+
+    def close(self):
+        "Close (quit) the connection"
+        self.connection.quit()
+        self.connection = None
+
+    def get_server(self) -> smtplib.SMTP:
+        "Connect and get the SMTP Server"
+        user = self.username
         password = self.password
         
         server = self.cls_smtp(self.host, self.port, **self.kws_smtp)
@@ -387,6 +454,11 @@ class EmailSender:
             server.login(user, password)
         return server
 
+    @property
+    def is_alive(self):
+        "bool: Check if there is a connection to the SMTP server"
+        return self.connection is not None
+
     def get_params(self, sender:str) -> Dict[str, Any]:
         "Get Jinja parametes passed to both text and html bodies"
         # TODO: Add receivers to params
@@ -467,3 +539,18 @@ class EmailSender:
             self.templates_html_table = jinja2.Environment(loader=jinja2.FileSystemLoader(html_table))
         if text_table is not None:
             self.templates_text_table = jinja2.Environment(loader=jinja2.FileSystemLoader(text_table))
+
+    def copy(self) -> 'EmailSender':
+        "Shallow copy EmailSender"
+        return copy(self)
+
+
+    @property
+    def user_name(self):
+        warnings.warn("Attribute user_name was renamed as username. Please use username instead.", FutureWarning)
+        return self.username
+
+    @user_name.setter
+    def user_name(self, user):
+        warnings.warn("Attribute user_name was renamed as username. Please use username instead.", FutureWarning)
+        self.username = user
\ No newline at end of file
